//       _________ __                 __
//      /   _____//  |_____________ _/  |______     ____  __ __  ______
//      \_____  \\   __\_  __ \__  \\   __\__  \   / ___\|  |  \/  ___/
//      /        \|  |  |  | \// __ \|  |  / __ \_/ /_/  >  |  /\___ |
//     /_______  /|__|  |__|  (____  /__| (____  /\___  /|____//____  >
//             \/                  \/          \//_____/            \/
//  ______________________                           ______________________
//                        T H E   W A R   B E G I N S
//         Stratagus - A free fantasy real time strategy game engine
//
//      (c) Copyright 2022 by Andrettin
//
//      This program is free software; you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation; only version 2 of the License.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program; if not, write to the Free Software
//      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//      02111-1307, USA.

#pragma once

#include "util/qunique_ptr.h"
#include "util/singleton.h"

namespace boost::asio {
	class io_context;
}

namespace wyrmgus {

class event_loop final : public singleton<event_loop>
{
public:
	event_loop();
	~event_loop();

	boost::asio::io_context &get_io_context() const
	{
		return *this->io_context;
	}

	void stop();

	void post(const std::function<void()> &function);

	void co_spawn(const std::function<boost::asio::awaitable<void>()> &function);

	[[nodiscard]]
	boost::asio::awaitable<void> await_ms(const uint64_t ms);

private:
	void run_io_context();

private:
	std::unique_ptr<boost::asio::io_context> io_context;
	qunique_ptr<QTimer> run_timer; //the timer for running the event loop again
};

}
